<?php


/**
 * Created by PhpStorm.
 * User: matheussilva
 * Date: 19/05/14
 * Time: 10:00
 */

namespace Folha\Logic\ObrigacaoLegal\RAIS;

use \Folha\Logic\Cadastros\Parametros\Estrutura;

use Geral\Entities\Estruturah;
use \Geral\Repositories\Estruturam;

use \Folha\Repositories\GpePessoa;

use \Folha\Entities\FpaFormrais;

use \Folha\Entities\FpaRaismagnet;

use \Folha\Entities\GpeVinculom;

use \Geral\Util\AcumuladorDeMensagens;

use \Geral\Util\Ambiente;

use \HCM\Util\DateUtil;

use \Geral\Util\NovoCodigo;

use \Geral\Util\TratamentoDeMensagens;

class GeraRais
{
    const CNPJ = 1;
    private $ano;
    private $nrorg;
    private $params;
    private $nrformrais;


    /**
     * Cria a classe para realizar as consultas no banco, tratar o resultado e
     * @param $nrorg
     * @param $ano
     * @param $nrformrais
     * @param $ret
     */
    public function __construct($nrorg, $ano, $nrformrais, $grupoIncidencia, $ret = 'N')
    {
        /**
         * @var $obj FpaFormrais
         */

        // DEFINE VALORES INICIAIS DAS PRINCIPAIS VARIÁVEIS
        $this->ano = $ano;
        $this->nrorg = $nrorg;
        $dtmescompetencia = "01/12/$ano";
        $this->nrformrais = $nrformrais;

        //BUSCA AS INFORMAÇÕES DO FILTRO
        $obj = FpaFormrais::getObjectByWhere('NRFORMRAIS = ? AND NRORG = ?', array($nrformrais, $nrorg));


        if($obj){
            //VERIFICA QUAIS AS ESTRUTURAS SE ENCAIXAM NO FILTRO
            $estrLegais = $this->buscaEstruturaLegal($obj, $nrorg, $ano);

            //DEFINE A LISTA DE ESTRUTURAS E VÍNCULOS QUE SE ENQUADRAM NOS FILTROS
            foreach($estrLegais as $estrLegal){
                $estruturas[] = $estrLegal['NRESTRUTLEGAL'];
                $vinculos[] = $estrLegal['NRVINCULOM'];
            }

            //REMOVE OS ITENS DUPLICADOS
            $estruturas = array_unique($estruturas);
            $vinculos = array_unique($vinculos);
            $estruturas = array_filter($estruturas);
            $vinculos = array_filter($vinculos);

            //VALIDAR INFORMAÇÃO
            $tipo_01 = Estruturam::getPreInformacao($ano, $nrorg, $estruturas);
            $valida_01 = new Prevalidar($tipo_01, 1);

            $tipo_02 = GpePessoa::getPreInformacao($ano, $nrorg, $estruturas);
            $valida_02 = new Prevalidar($tipo_02, 2);

            if (AcumuladorDeMensagens::getInstance()->existemMensagens()) {
                TratamentoDeMensagens::exibeMensagensAcumuladas();
//                return;
            }


            //SEPARA AS ESTRUTURAS EM VARIÁVEIS DE RAIS
            $rais = $this->separaArquivos($estruturas, $vinculos, $ano, $nrorg, $dtmescompetencia, $obj, $grupoIncidencia);

            //CRIA ARQUIVOS DE *.TXT E INSERE OS REGISTROS NO BANCO DE DADOS
            $this->criarMagnetico($rais, $nrorg, $obj);

        }
    }


    /**
     * Cria um arquivo de RAIS para cada empresa matriz
     *
     * @param $estruturas
     * @param $nrvinculom
     * @param $ano
     * @param $nrorg
     * @param $dtmescompetencia
     * @param $obj FpaFormrais
     * @return array
     */
    private function separaArquivos($estruturas, $nrvinculom, $ano, $nrorg, $dtmescompetencia, $obj, $grupoIncidencia)
    {
        $tipoInscricao = $obj->getNrhierarqorg();
        $tipoInscricaoResp = $obj->getIdtpinschierarq();
        $retificacao = $obj->getIdretifrais();
        $pessoa = $obj->getNrpessoarespons();
        $sindicato = $this->buscarCnpjEstrutura($obj->getNrestrutsind(), $nrorg); //Matheus Silva 04/03/2015

        //INICIALIZA AS MATRIZES PRINCIPAIS
        $matriz = array();
        $rais = array();

        //BUSCA QUAL A ESTRUTURA SUPERIOR
        foreach ($estruturas as $estrutura){
            $matriz[] = array(
                'FILIAL' => $estrutura,
                'MATRIZ' => Estruturam::getEstruturaSuperior($nrorg, $estrutura, $tipoInscricao, $dtmescompetencia)
            );
        }

        //REMOVE AS ESTRUTURAS QUE POSSIVELMENTE APARECERAM EM MAIS DE UM ARQUIVO
        $tam = count($matriz);
        for($c = 0; $c< $tam; $c++){
            for($d = 0; $d<$tam; $d++){
                if($matriz[$c]['FILIAL'] == $matriz[$d]['MATRIZ']){
                    $matriz[$d]['FILIAL'] = $matriz[$d]['MATRIZ'] = null;
                }
            }
        }

        //BUSCA OS DADOS DO RESPONSÁVEL E DADOS DO RETITIFCADOR
        foreach($matriz as $reg){
            if($reg['FILIAL'] != null){
                $estrutura = array( 0 => array('NRESTRUTURAMREL' => $reg['FILIAL']));
                $cnpjMatriz = Estruturam::getInfoMatriz($nrorg, $reg['FILIAL'], $ano);
                $cnpjMatriz = $cnpjMatriz[0]['CDCNPJESTRUT'];

                //PEGA OS DADOS DO RESPONSÁVEL PELA EMISSÃO DA RAIS
                $dadosResp = $this->pegaDadosResponsavel($nrorg, $ano, $reg['FILIAL'], $tipoInscricaoResp, $pessoa, $retificacao);

                //PEGA TODAS AS ESTRUTURAS FILIAIS DA MATRIZ.
                $estruturas = Estruturam::getEstruturaInferior($nrorg, $estrutura, $tipoInscricao, $dtmescompetencia);

                //CRIA UM OBJETO DE RAIS COM OS DADOS A SEREM GRAVADOS NO ARQUIVO E NO BANCO DE DADOS
                $rais[] = new Rais($estruturas, $nrvinculom, $nrorg, $this->ano, $cnpjMatriz, $dadosResp, $sindicato, $grupoIncidencia);
            }
        }
        return $rais;

    }


    /**
     * Gera o arquivo *.txt contendo a RAIS e salva o magnetico no banco.
     * @param $rais Rais
     * @param bool $inserir
     * @param $obj FpaFormRais
     * @throws \Exception
     */

    private function criarMagnetico($rais, $inserir = true, $obj)
    {
        $gravarBanco = $obj->getIdprocvinculo();
        $gerarMagnetico = $obj->getIdgerameiomag();
        $tam = count($rais);

        //GERA UM ARQUIVO PARA CADA RAIS E INSERE NO BANCO OS REGISTROS
        for($c = 0; $c<$tam; $c++){
            $cont = 1;

            // DEFINE AS INFORMAÇÕES DO TIPO 00 DO ARQUIVO
            $tipo00 = $rais[$c]->getTipo00();
            $tipo01 = $rais[$c]->getTipo01();
            $tipo02 = $rais[$c]->getTipo02();

            $file = $rais[$c]->zeroInicial($cont, 6);
            $file .= $tipo00['REGISTRO']."\n";
            $cont++;

            // DEFINE AS INFORMAÇÕES DO TOPO 01 DO ARQUIVO DA RAIS
            foreach($tipo01 as $linha_01){

                $file .= $rais[$c]->zeroInicial($cont, 6);
                $file .= $linha_01['REGISTRO']."\n";
                $cont++;
                try {
                    //ABRE UMA CONEXÃO PARA INSERIR OS ARQUIVOS BANCO
                    \Setforms2\Shared\Connection::getInstance()->beginTransaction();

                    //DEFINE AS INFORMAÇÕES DO TIPO 02 PARA SEREM SALVAS NO ARQUIVO
                    foreach ($tipo02 as $linha_02) {
                        if ($linha_02['NRESTRUTURAM'] == $linha_01['NRESTRUTURAM']) {
                            $file .= $rais[$c]->zeroInicial($cont, 6);
                            $file .= $linha_02['REGISTRO']."\n";

                            if ($gravarBanco == 'S') {
                                //SALVA AS INFORMAÇÕES NO BANCO DE DADOS
                                $this->salvarMagnetico($linha_02['REGISTRO'], $linha_02['CNPJ'], $linha_02['DTADMISSAO'], null, $linha_02['NRVINCULOM'], $linha_02['NRPISPASEP']);
                            }
                            $cont++;
                        }
                    }

                    //COMMIT NAS INFORMAÇÕES NO BANCO
                    \Setforms2\Shared\Registry::getEntityManager()->flush();
                    \Setforms2\Shared\Connection::getInstance()->commit();
                } catch (\Exception $e) {
                    \Setforms2\Shared\Connection::getInstance()->rollback();
                    throw $e;
                }
            }

            $file .= $rais[$c]->zeroInicial($cont, 6);
            $file .= $rais[$c]->getTipo09();

            if($gerarMagnetico == 'S'){
                $data = DateUtil::getDataAtual();
                $nome = "RAIS_".$tipo00['RAZAO_SOCIAL'].'_'.$data->format('dmYhis').".dec";
                $path = "..\\files\\RAIS";

                if (!file_exists($path)) {
                    mkdir($path, 0777, true);
                }

                $path = realpath($path);
                $caminho = $path.DIRECTORY_SEPARATOR.$nome;
                file_put_contents($caminho ,$file);

                $arquivo = new \TkFile(realpath($caminho)); //variável para o download do arquivo
                $arquivo->download();

            }
        }
    }

    /**
     * Busca os dados da empresa responsável pela emissão da RAIS
     *
     * @param $nrorg
     * @param $ano
     * @param $nrestrutura
     * @param $tipoInscricao
     * @param $nrpessoa
     * @param $retificacao
     * @return array
     */
    private function pegaDadosResponsavel($nrorg, $ano, $nrestrutura, $tipoInscricao, $nrpessoa, $retificacao)
    {
        $resp = array();

        //BUSCA AS INFORMAÇÕES DO RESPONÁVEL PELA EMISSÃO DA RAIS
        $info_estr = Estruturam::getInformacaoResponsavel($nrorg, $nrestrutura, $ano);
        $info_pess = GpePessoa::getRespRetificacao($nrorg, $nrpessoa);
        $info_estr = $info_estr[0];
        $info_pess = $info_pess[0];

        if($info_estr){
            $resp['RAZAO']        = $info_estr['RAZAO_SOCIAL'];
            $resp['LOGRADOURO']   = $info_estr['LOGRADOURO'];
            $resp['NUMERO']       = $info_estr['NUMERO'];
            $resp['COMPLEMENTO']  = $info_estr['COMPLEMENTO'];
            $resp['BAIRRO']       = $info_estr['BAIRRO'];
            $resp['CEP']          = $info_estr['CEP'];
            $resp['CDMUNICIPIO']  = $info_estr['CDMUNICIPIO'];
            $resp['NMMUNICIPIO']  = $info_estr['NMMUNCIPIO'];
            $resp['UF']           = $info_estr['ESTADO'];
            $resp['DDD']          = $info_estr['DDD'];
            $resp['TELEFONE']     = $info_estr['TELEFONE'];
        }
        if($retificacao == 'S'){
            $resp['RETIFICA_ID'] = 1;
            $resp['RETIFICA_DATA'] = DateUtil::getDataAtual()->format('d/m/Y');
        }else{
            $resp['RETIFICA_ID'] = 2;
            $resp['RETIFICA_DATA'] = '00000000';
        }
        if($info_pess){
            $resp['DATA_ARQUIVO']    = DateUtil::getDataAtual()->format('dmy');
            $resp['EMAIL']           = $info_pess['EMAIL'];
            $resp['NOME']            = $info_pess['NOME'];
            $resp['ESPACOS_1']       = '';
            $resp['CPF']             = $info_pess['CPF'];
            $resp['CREA']            = $info_pess['CREA'];
            $resp['DATA_NASCIMENTO'] = $info_pess['DATA_NASCIMENTO'];
            $resp['ESPACOS_2']       = '';
        }
        if($tipoInscricao == self::CNPJ){
            $resp['TIPO_INSCRICAO'] = 1;
            $resp['CNPJ']           = $info_estr['CNPJ'];
        }else{
            $resp['TIPO_INSCRICAO'] = 3;
            $resp['CNPJ']           = $info_estr['CEI'];
        }

        return $resp;
    }

    /**
     * Inserir o magnético no Banco de Dados.
     *
     * @param $linha
     * @param $cnpj
     * @param $admissao
     * @param $tipoInsc
     * @param $vinculo
     * @param $pis
     */
    private function salvarMagnetico($linha, $cnpj, $admissao, $tipoInsc, $vinculo, $pis)
    {
        $competencia = \FPA::getDtCompetencia();

        //TRATA A DATA DE ADMISSÃO
        if(!($admissao instanceof \DateTime)){
            $admissao = DateUtil::getDataDeString($admissao);
        }

        //TRATA A DATA DE COMPETENCIA
        if(!($competencia instanceof \DateTime)){
            $competencia = DateUtil::getDataDeString($competencia);
        }

        //CRIA UM OBJETO A SER SALVO NO BANCO
        $mag = new FpaRaismagnet();
        $mag->setNrraismagnet(NovoCodigo::geraCodigo(Ambiente::getNrorgTrabalho(), 'FPA_RAISMAGNET', 1, 22, true));
        $mag->setMescompetencia($competencia);
        $mag->setDslinharegistro($linha . '*');
        $mag->setCdinscestrut($cnpj);
        $mag->setDtadmissaovinc($admissao);
        $mag->setIdtpinscestrutu($tipoInsc);
        $mag->setNrvinculom($vinculo);
        $mag->setNrpispaseppes($pis);
        $mag->setNrorg(Ambiente::getNrorgTrabalho());
        $mag->setIdativo('S');
        $mag->setIdtiporegistro($this->nrformrais);
        $mag->setDtinclusao(DateUtil::getDataAtual());

        //SALVA UM OBJETO A SER COMITADO NO BANCO
        $mag->persist();
    }

    /**
     * Busca estruturas legais compativeis com o filtro.
     * @param $obj
     * @param $nrorg
     * @param $ano
     * @return array|bool
     */
    private function buscaEstruturaLegal($obj, $nrorg, $ano)
    {
        $params = array(
            'ORDEM' => $obj->getDsordem(),
            'NOME_INICIO' => $this->buscaNomeVínculo($nrorg, $obj->getNomeinicial()),
            'NOME_FINAL' => $this->buscaNomeVínculo($nrorg, $obj->getNomefinal()),
            'DTADMISSAO_INI' => $obj->getDtinicial(),
            'DTADMISSAO_FIM' => $obj->getDtfinal(),
            'TIPO_HIERARQUIA' => $obj->getNrhierarqorg(),
            'TIPO_INSCR' => $obj->getIdtpinschierarq(),
            'PESSOA_RESP' => $obj->getNrpessoarespons(),
            'ESTR_PESSOA_RESPONS' => $obj->getNrestrutrespons(),
            'TIPO_INSCR_RESPONSAVEL' => $obj->getIdtpinscrespons(),
            'ANO' => $ano
        );

        $this->params = $params;

        $estrLegais = \Folha\Repositories\GpeVinculom::buscaEstruturaLegal($nrorg, $params);

        if($estrLegais)
            return $estrLegais;

        return false;
    }

    /**
     * PEGA OS DADOS DO RESPONSÁVEL PELA RETIFICAÇÃO DA RAIS
     *
     * @param $nrorg
     * @param $nrpessoa
     * @return array|bool
     */
    public function pegaDadosRetif($nrorg, $nrpessoa)
    {
        return GpePessoa::getRespRetificacao($nrorg, $nrpessoa);
    }

    /**
     * Busca o CNPJ da estrutura com base na PK
     * @param int $nrestruturam
     * @param int $nrorg
     *
     * @return int
     * @throws \Exception
     */
    private function buscarCnpjEstrutura( $nrestruturam, $nrorg)
    {
        $where = "NRESTRUTURAM = :P_NRESTRUTURAM AND NRORG = :P_NRORG";
        $params = array(
            'P_NRESTRUTURAM' => $nrestruturam,
            'P_NRORG'        => $nrorg
        );
        /**
         * @var Estruturah $sindicato
         */
        $sindicato = Estruturah::getObjectByWhere($where, $params);
        return $sindicato->getCdcnpjestrut();
    }

    /**
     * Busca o vínculo pelo número de vínculo e pelo nrorg
     *
     * @param $nrorg
     * @param $nrvinculom
     *
     * @return String
     * @throws \Exception
     */
    private function buscaNomeVínculo($nrorg, $nrvinculom){
        try {
            $where = 'NRVINCULOM = :P_NRVINCULOM AND NRORG = :P_NRORG';
            $params = array(
                'P_NRVINCULOM' => $nrvinculom,
                'P_NRORG' => $nrorg
            );
            /**
             * @var GpeVinculom $result
             */
            $result = GpeVinculom::getObjectByWhere($where, $params);

            if ($result) {
                return $result->getNmvinculom();
            } else {
                return false;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
}